// ao32aa.inc
//
//  "Magic" header file for "The Art of ARM Assembly"
// Volume 2 that incoporates many of the features
// from Volume 1's aoaa.inc header file.
//
// This file is Raspberry Pi-specific!

#ifndef aoaa_inc
#define aoaa_inc 0
        
        
// 32-bit ARM
                     
                     .arch armv6                        
                     
// The .eabi_attribute values are magic numbers that
// are specific to the Raspberry Pi. Don't mess with
// these directives. 

                     .eabi_attribute 28, 1
                     .eabi_attribute 20, 1
                     .eabi_attribute 21, 1
                     .eabi_attribute 23, 3
                     .eabi_attribute 24, 1
                     .eabi_attribute 25, 1
                     .eabi_attribute 26, 2
                     .eabi_attribute 30, 6
                     .eabi_attribute 34, 1
                     .eabi_attribute 18, 4
         
// Standard ARM+Thumb syntax
         
                    .syntax unified

// Start in ARM mode (not THUMB)
        
                    .arm

// Support the FPU on the Raspberry Pi:
        
                    .fpu vfp
        

// Global declarations:
    
                    .global asmMain
                    .global getTitle
                    .global readLine
                    .global printf

        
        
// Gas doesn't have a .dword or .qword
// directive. Map .dword to .quad. 

                    .macro  .dword value:vararg
                    .quad   \value
                    .endm
                    
                    // Map .qword to .octa:
                    
                    .macro  .qword value:vararg
                    .octa   \value
                    .endm
    

// Macros for structure definitions
    
                    .set    __dir, 1

                    .macro  struct name, initialOffset=0
                    .set    __struct_offset, \initialOffset
                    .set    \name\().base, \initialOffset
                    .set    __dir, 1
                    
                    .if     \initialOffset > 0
                    .err    
                    .error  "struct offset must be negative or 0"
                    .exitm
                    .endif
                    
                    // Activate the declarations:
                    
                    __genDcls


                    // ends:
                    //
                    // Define the ends macro inside struct so
                    // it can only be used after a struct invocation.
                    //
                    // Generate name.size and name.offset constants
                    // specifying total structure size and the offset
                    // just beyond the last field.
                    //
                    // Also create a macro to be used to declare
                    // structure variables.

                    .macro ends name
                    .ifb    \name
                    .err
                    .error  "Expected structure name as argument"
                    .endif
                    
                    .set    \name\().size, __struct_offset-\name\().base
                    .set    \name\().offset, __struct_offset
                    
                    .macro  \name varName
                    .if     \name\().base < 0
                    .space  __struct_offset-(\name\().base)
                    .endif
\varName:
                    .if     __struct_offset > 0
                    .fill   __struct_offset
                    .endif

                    .endm   // \name macro
                    
                    
                    // Purge all the declaration macros
                    // after ends:
                    
                    __purgeDcls
                    
                    // ends is a one-shot macro that goes
                    // away once it is used:
                    
                    .purgem ends
                    .endm   // ends macro

                    .endm   // struct macro
                    

                    
// __genDcls:
//
// Invoked by struct, args, and locals in order to
// activate the declaration macros:

                    .macro  __genDcls
                    
                    .macro  salign size
                    .set    __salign, 0xFFFFFFFFFFFFFFFF - ((1 << \size)-1)
                    .set    __struct_offset, (__struct_offset + (1 << \size)-1) & __salign
                    .endm
                    
                    .macro  byte name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + \elements
                    .else
                    .set    __struct_offset, __struct_offset + \elements
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  hword name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*2)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*2)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  word name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*4)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*4)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  dword name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*8)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*8)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  qword name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offse, __struct_offset + ((\elements)*16)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*16)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  oword name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*32)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*32)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
                
                    .macro  single name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*4)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*4)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
        
                    .macro  double name, elements=1
                    .if     __dir > 0
                    .set    \name, __struct_offset
                    .set    __struct_offset, __struct_offset + ((\elements)*8)
                    .else
                    .set    __struct_offset, __struct_offset + ((\elements)*8)
                    .set    \name, -__struct_offset
                    .endif
                    .endm
                    
                    .endm   //__genDcls
                    
// __purgeDcls:
//
// When struct, args, or locals are done with the declarations,
// this macro purges all the declaration macros:

                    .macro  __purgeDcls
                    .purgem salign
                    .purgem byte
                    .purgem hword
                    .purgem word
                    .purgem dword
                    .purgem qword
                    .purgem oword
                    .purgem single
                    .purgem double
                    .endm

                    

                    

// Macro to emit a string that is padded with bytes
// so that it consumes a multiple of 4 bytes in memory

                    .macro   wastr theStr
                    .asciz   "\theStr"
                    .p2align 2
                    .endm
                    
                    
// Helper macro because \() is problematic in nested 
// macro definitions:

                    .macro  __append lbl, apnd, value
                    .set    \lbl\().\apnd, \value
                    .endm

                            
                    

/////////////////////////////////////////////////
// proc:
//
// Here's the actual proc macro.
// Usage:
//
//          proc name   // All defaults (private, ARM, no regs) 
//          proc name, {public | private | thumb}*

                    .set    __inProc, 0
                    .macro  proc pName:req, procArgs:vararg
                                        
                    .set    __isPublic, 0
                    .set    __isThumb, 0
                    
                    .if     __inProc
                    .err
                    .error  "Nested procedures are illegal"
                    .else
                    .set    __inProc, 1
                    .endif
                    
                    .irp    curArg, \procArgs
                    
                    .ifc    \curArg, public
                    .set    __isPublic, 1
                    .endif  // public
                    
                    .ifc    \curArg, private
                    .set    __isPublic, 0
                    .endif  // private
                    
                    .ifc    \curArg, thumb
                    .set    __isThumb, 1
                    .endif  // thumb
                    
                    .endr
                    
                    // Compute the number of registers this code
                    // has to preserve:
                    

                    .type   \pName, %function
                    
                // If "public" argument is present, emit
                // global statement.

                    .if     __isPublic
                    .global _\pName
                    .global \pName
                    .endif
        
                // If the "thumb" argument was present,
                // make this a thumb function:
                
                    .if     __isThumb
                    .align  1
                    .thumb
                    .thumb_func
                    .else
                    .align  2
                    .arm
                    .endif

\pName:
                    .if     __isThumb
                    .thumb
                    .thumb_func
                    .endif
_\pName:
                    .fnstart
                    
                /////////////////////////////////////////////
                // args:
                // Define the args macro within proc so 
                // that it is only usable inside a procedure:

                    .macro  args aName:req, initialOffset=8
                    .if     \aName != \pName
                    .err
                    .error  "args name does not match \pName"
                    .exitm
                    .endif
                    
                    .set    __dir, 1
                    .set    __struct_offset, \initialOffset
                    .set    __base, __struct_offset

                /////////////////////////////////////////////
                // enda:
                // Define enda inside args so that it is only
                // legal after an args declaration:

                    .macro  enda enda_Name:req
                    .if     \enda_Name != \pName
                    .err
                    .error  "enda name must match \pName"
                    .endif
                    
                    __append \enda_Name, argsize, __struct_offset-__base

                    // Purge all the declaration macros
                    // after seeing enda:
                    
                    __purgeDcls
                    
                    // Only one enda per args invocation:

                    .purgem enda
                    .endm   //enda
                    
                    // Activate all the declaration macros
                    // after seeing args:
                    
                    __genDcls
                    
                    .endm   //args
                    
                /////////////////////////////////////////////////
                // locals:
                // Define the locals macro only while a procedure
                // is active, include space for the preserved
                // registers when calculating offsets:

                    .macro  locals name
                    .set    __struct_offset, 0
                    .set    __dir, -1

                ///////////////////////////////////////////////////
                // endl:
                // Define endl only after seeing locals invocation:

                    .macro  endl name:req
                    .if     \name != \pName
                    .err
                    .error  "Expected locals name to match \pName"
                    .endif
                    __append \name, size, __struct_offset

                    // Purge all the declaration macros
                    // after seeing endl:
                    
                    __purgeDcls
                    
                    // Delete endl, making it a one-shot macro:
                    
                    .purgem endl
                    .endm   //endl

                    // Activate all the declaration macros
                    // after seeing locals:
                    
                    __genDcls
                    
                    .endm   //locals
                
                ////////////////////////////////////
                // Enter:
                // Assembly standard entry sequence.

                    .macro  enter localsSize:req, frame
                    
                    // Align the stack to a multiple of 8 bytes.
                    // Assuming dword-aligned stack on entry, this
                    // keeps the stack dword aligned (as per ABI).
                    // Note that we must subtract out the preserved
                    // register size because that portion of the
                    // stack automatically gets allocated by the
                    // push instruction:
                                       
                    .set    __enter_localsSize, ((\localsSize) + 7) & 0xFFFFFFF8
                    
                    // If a frame pointer (frame) register is specified
                    // as "none", then this activation register will not
                    // preserve the frame pointer on the stack. This code
                    // pushes R7 on the stack, not to preserve R7's value,
                    // but in order to keep the stack dword-aligned upon
                    // entry into the procedure body. Any register could
                    // be used in place of R7 here.
                    
                    .ifc    \frame, none
        
                    push    {r7, lr}
                    .save   {r7, lr}                    
                            
                    .else   // not .ifc \frame, none
                    
                    // If the frame pointer register was not
                    // specified, then default to FP (ARM) or
                    // R7 (Thumb):
                    
                    .ifb    \frame                  
                    .if     __isThumb
                    
                    push    {r7, lr}
                    .save   {r7, lr}                    
                    .setfp  r7, sp, #__enter_localsSize
                    mov     r7, sp

                    .else   //not __isThumb (is ARM)
                    
                    push    {fp, lr}
                    .save   {fp, lr}
                    .setfp  fp, sp, #__enter_localsSize
                    mov     fp, sp
                    
                    .endif  //__isThumb
                    
                    
                    .else   //not .ifb \frame
        
                    // Otherwise, use the frame register specified as
                    // an argument for the frame pointer:
                    
                    push    {\frame, lr}
                    .save   {\frame, lr}
                    .setfp  \frame, sp, #__enter_localsSize
                    mov     \frame, sp
                    .endif  //ifb
                    .endif  //ifc
                    

        
                // Allocate storage for local variables:
        
                    .if     __enter_localsSize > 0
        
                    .pad    #__enter_localsSize
                    
                    // If __enter_localsSize is 4000 or less,
                    // then only a single sub instruction
                    // is necessary:
                    
                    
                    .if     __enter_localsSize <= 4000
                    
                    sub     sp, sp, #__enter_localsSize
                    
                    
                    // If local size is greater than 4000 bytes,
                    // and less than 65000 bytes, then use
                    // R12 as a temporary register to drop
                    // the stack pointer down the appropriate
                    // amount.
                    
                    .elseif __enter_localsSize <= 65000
                    
                    .if     __isThumb
                    
                    mov     r12, r0
                    movs    r0, #__enter_localsSize >> 8
                    lsls    r0, r0, #8
                    add     sp, sp, r0
                    movs    r0, #__enter_localsSize & 0xff
                    add     sp, sp, r0
                    mov     r0, r12
                    
                    .else
                    
                    mov     r12, #__enter_localsSize >> 8
                    lsl     r12, r12, #8
                    add     sp, sp, r12
                    mov     r12, #__enter_localsSize & 0xff
                    add     sp, sp, r12
                    
                    .endif  // __isThumb
                    
                    .else  //__enter_localsSize > 65000
                    
                    // If the local size is too large (>64K)
                    // then generate an error.
                    
                    .err
                    .error  "Size operand for enter is too large"
                    
                    .endif  //__enter_localsSize < 4000 or 65000 
                    .endif  //__enter_localsSize > 0
        
                ///////////////////////////////////////////
                // leave:
                // Generate a different leave macro based
                // on the presence of the "frame" argument:
        
                    .ifc    \frame, none
        
                    // If the caller specified no frame register,
                    // clean up the stack frame by adding the
                    // localsSize value to SP; also removes
                    // dummy R7 value pushed on stack earlier:
        
                    .macro  leave
                    .if __enter_localsSize > 0
                    add     sp, #__enter_localsSize + 4
                    .endif
                    
                    pop     {pc}
                    
                    .purgem leave
                    .endm  

                    .else  // .ifc \frame
                    
                    .ifb    \frame
                    .if     __isThumb
                    
                    // If no frame pointer register was specified,
                    // default to r7:
        
                    .macro  leave
                    mov     sp, r7
                    
                    pop     {r7, pc}
                    .purgem leave
                    .endm  
                    
                    .else   //__isThumb 
                    
                    // If no frame pointer register was specified
                    // and this is ARM, default to fp:
        
                    .macro  leave
                    mov     sp, fp
                    pop     {fp, pc}
                    
                    .purgem leave
                    .endm
                    .endif  //__isThumb
                    
                    .else   //.ifb \frame

                    // If the caller specified a frame pointer
                    // registers to use (frame), then create
                    // a leave macro that will clean up the
                    // frame based on the frame pointer.
        
                    .macro  leave
                    mov     sp, \frame
                    pop     {\frame, pc}
                    
                    .purgem leave
                    .endm  
                    
                    .endif  // .ifb \frame
                    .endif  // .ifc 
        
                    .endm   // .macro enter


            ///////////////////////////////////////////////
            // endp:
            // End of a procedure, here.
            // Only allow endp while a procedure is active:

                    .macro  endp epName:req
                    .if     \epName\() != \pName
                    .err
                    .error "endp name does not match \pName"
                    .endif
                    .fnend
                    .size   \pName, .-\pName
                    
                    // We're leaving the procedure, 
                    // disable endp, enter, args, and locals

                    .purgem enter
                    .purgem locals
                    .purgem args
                    .purgem endp
                    .set    __inProc, 0
                    
                    .endm   //endp 
                    .endm   // proc
                    
                    

                              



                    

                             
                             
    
// getErrno-
//
// Retrieve's C errno value and returns
// it in R0 (PiOS only):
            
                    .extern __errno_location
                    .macro  getErrno
                    bl      __errno_location
                    ldr     r0, [r0]
                    .endm


// Opposite branches

        .macro  bnlt dest
        bge     \dest
        .endm
        
        .macro  bnle dest
        bgt     \dest
        .endm
        
        .macro  bnge dest
        blt     \dest
        .endm
        
        .macro  bngt dest
        ble     \dest
        .endm

        .macro  bnlo dest
        bhs     \dest
        .endm
        
        .macro  bnls dest
        bhi     \dest
        .endm
        
        .macro  bnhs dest
        blo     \dest
        .endm
        
        .macro  bnhi dest
        bls     \dest
        .endm
                    


#endif //ao32aa_inc
        
