// ao32aaLibPrivate.inc
//
// Private include file for use only by the
// integer to hexadecimal string conversion
// functions.

    #ifndef h32Private_inc
    #define h32Private_inc   1
    
// Constants that affect the CPU this code
// assembles for. Note that Cortex-M0+ code
// is in a separate library.

#define assembleForPi        0
#define assembleForTeensy4   1
#define assembleForTeensy3   2
#define assembleForCortexM4  3
#define assembleForDue       5
#define assembleForUnoR4     6

    
//**************************************************
//
// Define the following equate properly when
// building the library:

#define assembleFor assembleForPi
    
//**************************************************
    
// The following line must be changed as appropriate
// for the desired CPU/MPU:
    
    #if assembleFor == assembleForPi

    #include    "ao32aa.inc"
    .fpu    neon-vfpv4
    .arch   armv8-a
    
    #elif assembleFor == assembleForDue

    #include    "ao32aaDue.inc"
    
    #elif assembleFor == assembleForCortexM4

    #include    "ao32aaM4.inc"
    
    #elif assembleFor == assembleForTeensy3

    #include    "ao32aaTeensy3.inc"
    
    #elif assembleFor == assembleForTeensy3

    #include    "ao32aaTeensy3.inc"
    
    #elif assembleFor == assembleForUnoR4

    #include    "ao32aaUnoR4.inc"
    
    #elif assembleFor == assembleForTeensy4

    #include    "ao32aaTeensy4.inc"
    
    #else
    
    #error "assembleFor is not defined properly"
    
    #endif

// Public header file for the ao32aa library:
    
    #include    "ao32aaLib.inc"
    
// Common macros used by the conversion functions:

// Macro used by various integer to
// hex string functions. Emits
// two characters for a byte, with
// a leading '0' if value is in
// the range 0x0-0xf.

    .macro  emitByte, which, offset
     
    .if     \which == 0
    and     r2, r1, #0xff
    lsl     r2, r2, #1
    .else
    and     r2, r1, #0xff << (\which*8)
    lsr     r2, r2, #(\which*8) - 1
    .endif
    
    ldrh    r2, [r3, r2]
    strh    r2, [r0, #\offset]  
    .endm

    
// Macro used by h32ToStrB and h32ToStr_B.
//
// emitByteNZ (emit byte no zero).
//
//  If the specified byte (\which)
//  in R0 is in the range 0x10 to 0xff,
//  this macro emits two characters.
//  If the byte is in the range 0x1 to
//  0xF, this macro emits one character.
//  If the byte is zero, this macro
//  emits nothing unless \which is 0,
//  in which case it emits a single
//  '0' character.

    .macro  emitByteNZ, which, bifz
     
    lsl     r2, r1, #(3-\which)*8   // mov if which = 3
    lsr     r2, r2, #24
    lsls    r2, r2, #1
    beq     L\@
        
    // Emit two characters here.
    
    ldrh    r2, [r3, r2]
    strh    r2, [r0]    
    b       \bifz
L\@:
    .endm
    
        
    
// outPosns
//
// This macro will count the number
// of print positions required by the
// value in R0. It leaves the result in R4.

    .macro  outPosns
    clz     r4, r1
    cmp     r4, #32
    it      hs  
    subhs   r4, r4, #1
    lsrs    r4, r4, #2 
    rsbs    r4, r4, #8
    .endm   
    
    
// outPosns_
//
// This macro will count the number
// of print positions required by the
// value in R0, including an underscore
// if the number of digits exceeds four. 
// It leaves the result in R4.

    .macro  outPosns_
    clz     r4, r1
    
    // Zero has 32 leading 0s.
    // Need to set that to 31 to match 
    // other single-digit values
    
    cmp     r4, #32
    it      hs  
    subhs   r4, r4, #1
    
    // Divide bit count by 4
    // and subtract it from 8
    // to get the output positions.
    
    lsrs    r4, r4, #2 
    rsbs    r4, r4, #8
    
    // If more than four digits, add 1 for the
    // underscore character:
    
    cmp       r4, #5
    adc       r4, r4, #0
    .endm   
    
    #endif


