// expToBuf.S
//
// Converts an integer value to the floating-point
// exponent format. Used by both single- and double-
// precision floating-point output code.
																																		  
            #include    "ao32aaLibPrivate.inc"

            .text

//*************************************************************
//
// expToBuf
//
// Unsigned integer to buffer.
// Used to output up to 3-digit exponents.
//
// Inputs:
//
//    R0:   Points at buffer.
//    R1:   Unsigned integer to convert
//    R2:   Print width 1-3
        
			.align	2
            proc    expToBuf, thumb, public					
			push	{r1-r5, lr}	// Push R5 for stk alignment

            mov     r4, #10     // For division by 10

// Verify exponent digit count is in the range 1-3:

            cmp     r2, #1
            blo     badExp
            cmp     r2, #3
            bhi     badExp
            
// Verify the actual exponent will fit in the number of digits:


            cmp     r2, #2
            blo     oneDigit
            beq     twoDigits
            cmp     r2, #3
            bhi     badExp       //4+ digits, error
            cmp     r1, #1000
            bhs     badExp
            
// Convert 3-digit value to a string:

            udiv    r2, r1, r4      // R2=quotient
            mls    	r3, r2, r4, r1  // r3=remainder
            orr     r3, r3, #'0'
			strb	r3, [r0, #2]
            
            udiv    r1, r2, r4      // R1=quotient
            mls     r3, r1, r4, r2  // r3=remainder
            orr     r3, r3, #'0'
			strb	r3, [r0, #1]
            
            udiv    r2, r1, r4      // R2=quotient
            mls     r3, r2, r4, r1  // r3=remainder
            orr     r3, r3, #'0'
			strb	r3, [r0], #3
            b       outputExp
            
// Single digit is easy:

oneDigit:   
            cmp     r1, #10
            bhs     badExp
            
            orr     r3, r1, #'0'
			strb	r3, [r0], #1
            b       outputExp
            
// Convert value in the range 10-99 to a string
// containing two characters:

twoDigits:  
            cmp     r1, #100
            bhs     badExp
            
            udiv    r2, r1, r4      // R2=quotient
            mls     r3, r2, r4, r1  // r3=remainder
            orr     r3, r3, #'0'
            strb	r3, [r0, #1]      
            
            udiv    r1, r2, r4      // R1=quotient
            mls     r3, r1, r4, r2  // r3=remainder
            orr     r3, r3, #'0'
            strb	r3, [r0], #2

outputExp:
			mov		r3, #0
			strb	r3, [r0]
			adds	r0, r0, #0		//clc = no error
			pop		{r1-r5, pc}
            
badExp:
            mov     r0, #-1      	// Value out of range
			adds	r1, r0, r0		// stc = error
			pop		{r1-r5, pc}
            endp    expToBuf
