// h32PadStr.S

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2

/////////////////////////////////////////////////////
//            
// h32PadStr
//
// Emits padding characters for an output string.
//
//  R0: Pointer to buffer for the fill characters.
//  R3: Fill character to emit.
//  R4: Number of fill characters to emit.
//
// Leaves R0 pointing beyond the last fill character
// emited by this code.
//
// Assumption: R4 is greater than zero.

    proc        h32PadStr, public
    push        {r5, lr}

    // If there are four or more characters to
    // output, then optimize things a bit by
    // emitting four characters at a time:
        
    cmp         r4, #4
    blo         fillLp2
    
    ldr         r5, =0x01010101
    mul         r5, r5, r3      // Copy char to all four bytes
fill4Lp2:
    str         r5, [r0], #4
    subs        r4, r4, #4
    cmp         r4, #4  
    bhs         fill4Lp2

    // See if 1 to 3 bytes remain to output:
    
    cmp         r4, #0
    bne         fillLp2
    
    // Return if no more bytes to output:
    
    pop         {r5, pc}
    
    // For 1 to 3 characters, emit them one at a time.
            
fillLp2:
    strb        r3, [r0], #1
    subs        r4, r4, #1
    bne         fillLp2
    
noFill2:
    pop         {r5, pc}
    endp        h32PadStr        
        
        
