// h32ToStrW.S
//
// Formatted hexadecimal conversion function that
// emits all bytes except HO bytes containing zeros.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2

///////////////////////////////////////////
//
// h32ToStrW
//
// Converts the integer in R1 a hexadecimal
// string. Output width is always a multiple
// of four characters (for example,
// 0000 or 10000000). If the HO word is zero,
// this function emits four digits, else it
// emits eight digits.
//
// Macro notes:
//
//	emitBytLO changes R2 and it assumes that
//	R3 contains the address of h32LookupTable256.
		
	#define preserve   push {r2, r3}
	#define restore    pop  {r2, r3}

    proc        h32ToStrW, thumb, public
    preserve
    ldr         r3, =h32LookupTable256
    ldr         r2, =0x10000
    cmp         r1, r2
    blo         LOhword
    
	// Emit all four bytes here:
	
    emitByte  	3, 0
    emitByte  	2, 2
    emitByte  	1, 4
    emitByte  	0, 6
    
    // Zero-terminate the string
    
    movs        r2, #0
    strb        r2, [r0, #8] 
    restore
	bx			lr

	// Emit the LO two bytes here:
	
LOhword:
    emitByte  	1, 0
    emitByte  	0, 2
    movs        r2, #0
    strb        r2, [r0, #4] 
    restore
	bx			lr
    endp        h32ToStrW
	

