// h32ToStrB.S
//
// Formatted hexadecimal conversion function that
// emits all bytes except HO bytes containing zeros.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2

///////////////////////////////////////////
//
// h32ToStr_B
//
// Converts the integer in R1 a hexadecimal
// string with a possible single leading zero 
// for the highest-order non-zero byte.
// Will inject an underscore between bytes
// 1 and 2 if the value in R0 is greater than
// 0xffff.

	#define preserve	push {r2-r3}
	#define restore		pop	 {r2-r3}

    proc        h32ToStr_B, thumb, public
    preserve
    ldr         r3, =h32LookupTable256
    
    // The following handle leading zeros,
    // branch off to nonZero* if they emit
    // a non-zero value.
    
    emitByteNZ  3, nonZero2
    emitByteNZ  2, nonZero1
    emitByteNZ  1, nonZero0

    // LO byte is emitted as two chars, always:
    
    emitByte  	0, 0
	mov			r2, #0
	strb		r2, [r0, #2]
    restore
	bx			lr

    // Come down here when the code has encountered
    // a non-zero output value. From this point on
    // the code must emit leading zeros:
    
nonZero2:
    emitByte  	2, 2
	mov			r2, #'_'
	strb		r2, [r0, #4]
	emitByte	1, 5
	emitByte	0, 7
	mov			r2, #0
	strb		r2, [r0, #9]
    restore
	bx			lr
	
nonZero1:
	mov			r2, #'_'
	strb		r2, [r0, #2]
	emitByte	1, 3
	emitByte	0, 5
	mov			r2, #0
	strb		r2, [r0, #7]
    restore
	bx			lr

nonZero0:
	emitByte	0, 2
	mov			r2, #0
	strb		r2, [r0, #4]
    restore
	bx			lr

    endp        h32ToStr_B

