// h32ToStr_NLZ.S
//
// Convert an integer to hexadecimal string
// with no leading zeros.
//
// Injects an underscore between digits 3 and 4.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2
    
///////////////////////////////////////////
//
// h32ToStr_NLZ
//
// Converts the integer in R1 a hexadecimal
// string with no leading zeros and no
// underscores.

	#define 	preserve push {r2-r3}
	#define 	restore  pop  {r2-r3}

	.macro		jmpTbl location
	.rept		4
	.byte		(\location-tb)/2
	.endr
	.endm

    proc        h32ToStr_NLZ, thumb, public
    preserve
    
    ldr         r3, =h32LookupTable256  
    
    // Switch statement:
    
    clz     r2, r1      // Count leading zeros
    tbb     [pc, r2]    // Switch to specific case 
tb:
	jmpTbl	eightDigits
	jmpTbl	sevenDigits
	jmpTbl	sixDigits
	jmpTbl	fiveDigits
	jmpTbl	fourDigits
	jmpTbl	threeDigits
	jmpTbl	twoDigits
	jmpTbl	oneDigit
	jmpTbl	oneDigit	// Special case for 0
	

// Emit strings containing an even number of digits here:

eightDigits:    
    emitByte  	3, 0
    emitByte  	2, 2
	mov			r2, #'_'
	strb		r2, [r0, #4]
    emitByte  	1, 5
    emitByte  	0, 7
	mov			r2, #0
	strb		r2, [r0, #9]
	restore
	bx			lr
	
sixDigits:
    emitByte  	2, 0
	mov			r2, #'_'
	strb		r2, [r0, #2]
    emitByte  	1, 3
    emitByte  	0, 5
	mov			r2, #0
	strb		r2, [r0, #7]
	restore
	bx			lr

fourDigits:
    emitByte  	1, 0
    emitByte  	0, 2
	mov			r2, #0
	strb		r2, [r0, #4]
	restore
	bx			lr

twoDigits:
    emitByte  	0, 0
	mov			r2, #0
	strb		r2, [r0, #2]
	restore
	bx			lr

// Handle odd length (1-7) digits down here:

oneDigit:   
    ldr         r3, =h32LookupTable16
	ands		r2, r1, #0xf
    ldrb        r2, [r3, r2]
    strb        r2, [r0]
	mov			r2, #0
	strb		r2, [r0, #1]
    restore
	bx			lr
    
threeDigits:
    ldr     	r3, =h32LookupTable16
    lsrs    	r2, r1, #8
    ands    	r2, r2, #0xf
    ldrb    	r2, [r3, r2]
    strb    	r2, [r0]

    ldr         r3, =h32LookupTable256
    emitByte 	0, 1
	mov			r2, #0
	strb		r2, [r0, #3]
    restore
	bx			lr

fiveDigits:
    ldr     	r3, =h32LookupTable16
    lsrs    	r2, r1, #16
    ands    	r2, r2, #0xf
    ldrb    	r2, [r3, r2]
    strb    	r2, [r0]
	
	mov			r2, #'_'
	strb		r2, [r0, #1]

    ldr         r3, =h32LookupTable256
    emitByte  	1, 2
    emitByte  	0, 4
	mov			r2, #0
	strb		r2, [r0, #6]
    restore
	bx			lr

sevenDigits:
    ldr     	r3, =h32LookupTable16
    lsrs    	r2, r1, #24
    ands    	r2, r2, #0xf
    ldrb    	r2, [r3, r2]
    strb    	r2, [r0]

    ldr         r3, =h32LookupTable256
    emitByte	2, 1
	mov			r2, #'_'
	strb		r2, [r0, #3]
    emitByte	1, 4
    emitByte	0, 6
	mov			r2, #0
	strb		r2, [r0, #8]
    restore
	bx			lr
	
	endp		h32ToStr_NLZ

