// h32ToStrW_.S
//
// Formatted hexadecimal conversion function that
// emits all bytes except HO bytes containing zeros.

    #include    "ao32aaLibPrivate.inc"

    .text
    .align      2

///////////////////////////////////////////
//
// h32ToStr_W
//
// Converts the integer in R1 a hexadecimal
// string. If there are more than four output
// digits, this function injects an underscore
// between the LO and HO hword outputs.
// Output width is always four or nine (for example,
// 0000 or 1000_0000). If the HO hword is zero,
// this function emits four digits, else it
// emits eight digits plus an underscore.
//
// Macro notes:
//
//	emitBytLO changes R2 and it assumes that
//	R3 contains the address of h32LookupTable256.
		
	#define preserve   push {r2, r3}
	#define restore    pop  {r2, r3}

    proc        h32ToStr_W, thumb, public
    preserve
    ldr         r3, =h32LookupTable256
    ldr         r2, =0x10000
    cmp         r1, r2
    blo         LOhword
    
	// Emit all four bytes here:
	
    emitByte  	3, 0
    emitByte  	2, 2
	
	// Emit an underscore between
	// the HO and LO hwords:
	
	movs	r2, #'_'
	strb	r2, [r0, #4]
	
	// Emit the LO two bytes here:
	
    emitByte  	1, 5
    emitByte  	0, 7
    
    // Zero-terminate the string and return:
    
    movs        r2, #0
    strb        r2, [r0, #9] 
    restore
	bx			lr
	
// Handle the case where the HO hword is zero:	
	
LOhword:
    emitByte  	1, 0
    emitByte  	0, 2
    
    // Zero-terminate the string
    
    movs        r2, #0
    strb        r2, [r0, #4] 
    restore
	bx			lr
    endp        h32ToStr_W
