// i32FmtStr.S
//
// i32FmtStr converts a signed 32-bit
// integer in R1 to a string of decimal
// digits, storing the string in the
// location pointed at by R0.
// This formatting version also outputs
// the string using a minimum field width
// and using a supplied fill character
// (if the number of digits is less than
// the field width).


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2
    
//////////////////////////////////////
//
// i32FtmStr
//
//	Converts a 32-bit signed integer
// to a string of decimal digits, with
// a minimum field width and a fill
// character (if the output string of
// digits will be less than the minimum
// field width).
//
// Inputs:
//	R0: Pointer to buffer to receive
//		the string (must be minimum
//		width + 1 bytes long, or 11,
//		whichever is greater).
//	R1:	Value to convert.
//	R2: Minimum field width
//  R3: Fill character (byte)

		proc	i32FmtStr, thumb, public
		cmp		r1, #0
		blt		negativeNumber
		
		b		u32FmtStr

negativeNumber:
		push	{r0-r4, lr}
		
		// Compute the actual number of
		// digits in the number (u32Digits
		// returns result in R4):
		
		neg		r1, r1
		bl		u32Digits
		add		r4, r4, #1		// Include '-'
		cmp		r4, r2
		bhs		noFillChars
		
// If the field width is greater than the number
// of digits in the converted value, then this
// code has to emit fill characters to make up
// the difference.
//
// In theory, this code could be sped up by
// writing four bytes at a time as long
// as there are still four fill characters
// to output. However, most widths are
// probably 12 characters or less and
// the overhead to output a word at a time
// probably would remove any benefit of
// doing this.

fillLp:
		strb	r3, [r0], #1
		subs	r2, r2, #1
		cmp		r2, r4
		bhi		fillLp
		
// At this point, R1 points at the first
// character position in the string.
// Call u32ToStrNoDigits to do the conversion.

noFillChars:		
		bl		u32ToStrNoDigits
		mov		r1, #'-'	// Emit the minus sign
		strb	r1, [r0]
		pop		{r0-r4, pc}
		endp	i32FmtStr
		