// i64FmtStr.S
//
// i64FmtStr converts a signed 64-bit
// integer in R2:R1 to a string of decimal
// digits, storing the string in the
// location pointed at by R0.
// This formatting version also outputs
// the string using a minimum field width
// and using a supplied fill character
// (if the number of digits is less than
// the field width).


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2
    
//////////////////////////////////////
//
// i64FtmStr
//
//	Converts a 64-bit signed integer
// to a string of decimal digits, with
// a minimum field width and a fill
// character (if the output string of
// digits will be less than the minimum
// field width).
//
// Inputs:
//	R0-		Pointer to buffer to receive
//			the string (must be minimum
//			width + 1 bytes long, or 21,
//			whichever is greater).
//	R2:R1-	Value to convert.
//	R3-		Minimum field width
//  R4- 	Fill character (byte)

		proc	i64FmtStr, thumb, public
		cmp		r2, #0
		blt		negativeNumber
		beq		gotoU32
		b		u64FmtStr
		
// If the HO word is zero,
// then use u32FmtStr to do
// the work because it is much
// faster.
		
gotoU32:
		push	{r3, r4, lr}
		mov		r2, r3		// Put parameters in the
		mov		r3, r4		// right locations
		bl		u32FmtStr
		pop		{r3, r4, pc}

negativeNumber:
		push	{r0-r5, lr}
		
		// Compute the actual number of
		// digits in the number (u64Digits
		// returns result in r5):

		mov		lr, #0			// 64-bit negation
		subs	r1, lr, r1
		sbc		r2, lr, r2		

		bl		u64Digits
		add		r5, r5, #1		// Include '-'
		cmp		r5, r3
		bhs		noFillChars
		
// If the field width is greater than the number
// of digits in the converted value, then this
// code has to emit fill characters to make up
// the difference.
//
// In theory, this code could be sped up by
// writing four bytes at a time as long
// as there are still four fill characters
// to output. However, most widths are
// probably 12 characters or less and
// the overhead to output a word at a time
// probably would remove any benefit of
// doing this.

fillLp:
		strb	r4, [r0], #1
		subs	r3, r3, #1
		cmp		r3, r5
		bhi		fillLp
		
// At this point, R1 points at the first
// character position in the string.
// Call u64ToStrNoDigits to do the conversion.

noFillChars:
		add		r0, r0, r5
		mov		lr, #0
		strb	lr, [r0], #-1		
		bl		u64ToBufRev
		mov		r1, #'-'	// Emit the minus sign
		strb	r1, [r0]
		pop		{r0-r5, pc}
		endp	i64FmtStr
		