// i64ToBuf.S
//
// i64ToBuf converts a signed 64-bit
// integer in R2:R1 to a string of decimal
// digits.


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2
    
	

////////////////////////////////////////////////
//
//  i64ToBuf
//
//	Converts an integer to string of decimal
// digits using the minimum number of output
// positions.
//
// Inputs:
//	R0-		Pointer to the buffer.
//	R2:R1-	Number to convert.
//
// Output:
//	R0: points at the first byte after
//		the string of converted digits.
//
// Does not preserve any registers! Caller is
// resposible for saving R0-R4, if necessary.
//
// Note that this procedure does not zero-terminate
// the string.


    proc    i64ToBuf, thumb, public
	cmp		r2, #0
	blt		negativeNumber
	
	// If the number is positive, let u64ToBuf
	// do all the work:
	
	b		u64ToBuf
	
negativeNumber:
	push	{lr}	
	
	// Compute the number of output positions this
	// number requires, and point R1 at the end of
	// the buffer where the least significant digit
	// will go:

	mov		r5, #0
	subs	r1, r5, r1
	sbc		r2, r5, r2
	bl		u64Digits
	add		r0, r0, r5	// Point at end of buffer+1
	add		r0, r0, #1	// Will be ultimiate end 
	push	{r0}		// Preserve this.
	sub		r0, r0, #1	// Last digit goes here.
		
	// Convert the negated integer to digits.
	// Note: u64ToBufRev does not mess with R4.
	
	bl		u64ToBufRev
	
	// Inject a '-' at the beginning of the string
	// because the number was negative:

	mov		r1, #'-'
	strb	r1, [r0]
	pop		{r0,pc}
	endp	i64ToBuf




