// u32ToBuf.S
//
// u32ToBuf converts an unsigned 32-bit
// integer in R0 to a string of decimal
// digits.


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2
    
	

////////////////////////////////////////////////
//
//  u32ToBuf
//
//	Converts an integer to string of decimal
// digits using the minimum number of output
// positions.
//
// Inputs:
//	R0: pointer to the buffer.
//	R1: number to convert.
//
// Output:
//	R0: points at the first byte after
//		the string of converted digits.
//
// Does not preserve any registers! Caller is
// resposible for saving R0-R4, if necessary.
//
// Note that this procedure does not zero-terminate
// the string.


    proc    u32ToBuf, thumb, public
	push	{lr}
	
	// Compute the number of output positions this
	// number requires, and point R1 at the end of
	// the buffer where the least significant digit
	// will go:
		
	bl		u32Digits
	add		r0, r0, r4	// Point at end of buffer+1
	push	{r0}		// Preserve R0
	sub		r0, r0, #1	// Point at last char posn
	
	// Convert the string to digits:
	
	bl		u32ToBufRev
	
	// Because R1 was pushed last, it is at the
	// lowest address on the stack and the following
	// pop properly puts TOS into R1:
	
	pop		{r0, pc}
	endp	u32ToBuf

////////////////////////////////////////////////
//
//  u32ToBufRev
//
//	Converts an integer to string of decimal
// digits using the minimum number of output
// positions.
//
// Inputs:
//	R0: pointer to the end of the buffer
//      where the least significant digit
//      will go.
//	R1: number to convert
//
// Output:
//	R0: points at the first byte before
//		the string of converted digits.
//
//
// Does not preserve any registers! Caller
// is responsible for preserving R0-R4, as
// needed.
//
// Note that this procedure does not zero-terminate
// the string.

	// Utility macro (just to save typing):
	
    .macro  divmod dest, src
    udiv    \dest, \src, r2
    mov     r4, \dest
    mls     r4, r4, r2, \src
    orr     r4, r4, #'0'
    strb    r4, [r0], #-1
    cmp     \dest, #0
    beq     uDone
    .endm
    
	
	// Conversion function:
	
    proc    u32ToBufRev, thumb, public

	// R2 contains the constant used
	// by division and mod calculations:
	
    mov     r2, #10
    
	// Convert each digit, from LO to digit 9:
	
    divmod  r3, r1
    divmod  r1, r3
    divmod  r3, r1
    divmod  r1, r3
    divmod  r3, r1
    divmod  r1, r3        
    divmod  r3, r1        
    divmod  r1, r3
    divmod  r3, r1
    
    // Handle 10th digit specially
    
    udiv    r1, r3, r2
    mls     r4, r1, r2, r3
    orr     r4, r4, #'0'
    strb    r4, [r0], #-1

uDone:
    bx		lr 
    endp    u32ToBufRev



