// u64FmtStr.S
//
// u64FmtStr converts an unsigned 64-bit
// integer in R2:R1 to a string of decimal
// digits, storing the string in the
// location pointed at by R0.
// This formatting version also outputs
// the string using a minimum field width
// and using a supplied fill character
// (if the number of digits is less than
// the field width).


	#include	"ao32aaLibPrivate.inc"

	.text
	.align  2


//////////////////////////////////////
//
// u64FmtStr
//
//	Converts a 64-bit unsigned integer
// to a string of decimal digits, with
// a minimum field width and a fill
// character (if the output string of
// digits will be less than the minimum
// field width).
//
// Inputs:
//	R0-		Pointer to buffer to receive
//			the string (must be minimum
//			width + 1 bytes long, or 11,
//			whichever is greater).
//	R2:R1-	Value to convert.
//	R3-		Minimum field width
//  R4-		Fill character (byte)

		proc	u64FmtStr, thumb, public
		push	{r0-r5, lr}
		
		// Compute the actual number of
		// digits in the number (u64Digits
		// returns result in R5):
		
		bl		u64Digits
				
		// See if function needs to 
		// add padding bytes:
		
		cmp		r5, r3
		bhi		noFillChars
		
// If the field width is greater than the number
// of digits in the converted value, then this
// code has to emit fill characters to make up
// the difference.
//
// In theory, this code could be sped up by
// writing four bytes at a time as long
// as there are still four fill characters
// to output. However, most widths are
// probably 12 characters or less and
// the overhead to output a word at a time
// probably would remove any benefit of
// doing this.

fillLp:
		strb	r4, [r0], #1
		subs	r3, r3, #1
		cmp		r3, r5
		bhi		fillLp
		
// At this point, R1 points at the first
// character position in the string.
// Call u64ToBufRev to do the conversion.

noFillChars:
		add		r0, r5			// Point at end of buffer
		mov		lr, #0			// Zero terminate
		strb	lr, [r0], #-1
		bl		u64ToBufRev		// Convert digits
		
		pop		{r0-r5, pc}
		endp	u64FmtStr
		